﻿// Stammbuch Thumbnailer modified by Daniel Kirstenpfad
// which was originally created by Dmitry Robsman
// and modified as Photo Album handler by Bertrand Le Roy
// with the participation of David Ebbo
// http://www.gotdotnet.com/Workspaces/Workspace.aspx?id=1eceb66c-3bd0-4cdd-87d5-3fa0f4742032

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Runtime.InteropServices;

namespace StammbuchService
{
    /// <summary>
    /// Static methods and constants for use by the Album class.
    /// </summary>
    public class Thumbnailer
    {
        private static readonly Color BackGround = Color.Black;                                 /// The background color of the image thumbnails and previews.
        private static readonly Color BorderColor = Color.Beige;                                /// The color used to draw borders around image thumbnails and previews.
        private static readonly float ThumbnailBorderWidth = 0;                                 /// The width in pixels of the thumbnail border.
        private static readonly float PreviewBorderWidth = 0;                                   /// The width in pixels of the image previews.
        private static readonly Color UpFolderBorderColor = Color.FromArgb(90, Color.Black);    /// The color of the shadow that's drawn around up folder stacked image thumbnails.
        private static readonly float UpFolderBorderWidth = 2;                                  /// The width in pixels of the shadow that's drawn around up folder stacked image thumbnails.
        private static readonly Color UpArrowColor = Color.FromArgb(90, Color.Black);          /// The color of the arrow that's drawn on up folder thumbnails.
        private static readonly float UpArrowWidth = 4;                                         /// The color of the arrow that's drawn on up folder thumbnails.
        private static readonly float UpArrowSize = 0.125F;                                     /// The relative size of the arrow that's drawn on up folder thumbnails.
        private static readonly int UpFolderStackHeight = 3;                                    /// The number of images on the up folder thumbnail stack.
        private static readonly long ThumbnailJpegQuality = 75L;                                /// The quality (between 0 and 100) of the thumbnail JPEGs.
        private const int MemoryCacheSlidingDuration = 10;                                      /// If using memory cache, the duration in minutes of the sliding expiration.

        /// <summary>
        /// Creates a Bitmap object from a file.
        /// </summary>
        /// <param name="imageFile">The path ofthe image.</param>
        /// <returns>The bitmap object.</returns>
        public Bitmap LoadOriginalImage(string imageFile) {
            return (Bitmap)Bitmap.FromFile(imageFile, false);
        }

        /// <summary>
        /// Creates a reduced Bitmap from a full-size Bitmap.
        /// </summary>
        /// <param name="originalImage">The full-size bitmap to reduce.</param>
        /// <param name="size">The size in pixels of a square bounding the reduced bitmap.</param>
        /// <param name="thumbnail">True if the reduced image is a thumbnail.</param>
        /// <returns>The reduced Bitmap.</returns>
        public Bitmap CreateNewImage(Bitmap originalImage, int size, bool thumbnail)
        {
            int originalWidth = originalImage.Width;
            int originalHeight = originalImage.Height;
            int width, height;
            int drawXOffset, drawYOffset, drawWidth, drawHeight;

            if (size > 0 && originalWidth >= originalHeight && originalWidth > size) {
                width = size;
                height = Convert.ToInt32((double)size * (double)originalHeight / (double)originalWidth);
            }
            else if (size > 0 && originalHeight >= originalWidth && originalHeight > size) 
            {
                width = Convert.ToInt32((double)size * (double)originalWidth / (double)originalHeight);
                height = size;
                //height = Convert.ToInt32((double)size * (double)originalWidth / (double)originalHeight);
                //width = size;

            }
            else 
            {
                width = originalWidth;
                height = originalHeight;
            }

            drawXOffset = 0;
            drawYOffset = 0;
            drawWidth = width;
            drawHeight = height;

            if (thumbnail) {
                width = Math.Max(width, size);
                height = Math.Max(height, size);
                drawXOffset = (width - drawWidth) / 2;
                drawYOffset = (height - drawHeight) / 2;
            }

            Bitmap newImage = new Bitmap(width, height);
            Graphics g = Graphics.FromImage(newImage);
            g.InterpolationMode = InterpolationMode.HighQualityBicubic;
            g.SmoothingMode = SmoothingMode.AntiAlias;

            float borderWidth = thumbnail ? ThumbnailBorderWidth : PreviewBorderWidth;
            g.FillRectangle(new SolidBrush(BackGround), 0, 0, width, height);
            Pen BorderPen = new Pen(BorderColor);
            BorderPen.Width = borderWidth;
            BorderPen.LineJoin = LineJoin.Round;
            BorderPen.StartCap = LineCap.Round;
            BorderPen.EndCap = LineCap.Round;
            g.DrawRectangle(BorderPen,
                drawXOffset + borderWidth / 2,
                drawYOffset + borderWidth / 2,
                drawWidth - borderWidth,
                drawHeight - borderWidth);

            g.DrawImage(originalImage,
                drawXOffset + borderWidth,
                drawYOffset + borderWidth,
                drawWidth - 2 * borderWidth,
                drawHeight - 2 * borderWidth);
            g.Dispose();
            return newImage;
        }

        /// <summary>
        /// Creates the thumbnail Bitmap for a folder.
        /// </summary>
        /// <param name="isParentFolder">True if the up arrow must be drawn.</param>
        /// <param name="folder">The path of the folder.</param>
        /// <param name="size">The size in pixels of a square bounding the thumbnail.</param>
        /// <returns>The thumbnail Bitmap.</returns>
        public Bitmap CreateFolderImage(bool isParentFolder, string folder, int size)
        {
            Bitmap newImage = new Bitmap(size, size);
            Graphics g = Graphics.FromImage(newImage);
            g.InterpolationMode = InterpolationMode.HighQualityBicubic;
            g.SmoothingMode = SmoothingMode.AntiAlias;

            Random rnd = new Random();
            List<string> imagesToDraw = new List<string>();
            int nbFound;
            string[] images = Directory.GetFiles(folder, "default.jpg", SearchOption.AllDirectories);
            for (nbFound = 0; nbFound < Math.Min(UpFolderStackHeight, images.Length); nbFound++) {
                imagesToDraw.Insert(0, images[nbFound]);
            }
            if (nbFound < UpFolderStackHeight) {
                images = Directory.GetFiles(folder, "*.jpg", SearchOption.AllDirectories);
                for (int i = 0; i < Math.Min(UpFolderStackHeight - nbFound, images.Length); i++) {
                    imagesToDraw.Insert(0, images[rnd.Next(images.Length)]);
                }
            }
            int drawXOffset = size / 2;
            int drawYOffset = size / 2;
            double angleAmplitude = Math.PI / 10;
            int imageFolderSize = (int)(size / (Math.Cos(angleAmplitude) + Math.Sin(angleAmplitude)));

            foreach (string folderImagePath in imagesToDraw) {
                Bitmap folderImage = new Bitmap(folderImagePath);

                int width = folderImage.Width;
                int height = folderImage.Height;
                if (imageFolderSize > 0 && folderImage.Width >= folderImage.Height && folderImage.Width > imageFolderSize) {
                    width = imageFolderSize;
                    height = imageFolderSize * folderImage.Height / folderImage.Width;
                }
                else if (imageFolderSize > 0 && folderImage.Height >= folderImage.Width && folderImage.Height > imageFolderSize) {
                    width = imageFolderSize * folderImage.Width / folderImage.Height;
                    height = imageFolderSize;
                }

                Pen UpFolderBorderPen = new Pen(new SolidBrush(UpFolderBorderColor), UpFolderBorderWidth);
                UpFolderBorderPen.LineJoin = LineJoin.Round;
                UpFolderBorderPen.StartCap = LineCap.Round;
                UpFolderBorderPen.EndCap = LineCap.Round;

                double angle = (0.5 - rnd.NextDouble()) * angleAmplitude;
                float sin = (float)Math.Sin(angle);
                float cos = (float)Math.Cos(angle);
                float sh = sin * height / 2;
                float ch = cos * height / 2;
                float sw = sin * width / 2;
                float cw = cos * width / 2;
                float shb = sin * (height + UpFolderBorderPen.Width) / 2;
                float chb = cos * (height + UpFolderBorderPen.Width) / 2;
                float swb = sin * (width + UpFolderBorderPen.Width) / 2;
                float cwb = cos * (width + UpFolderBorderPen.Width) / 2;

                g.DrawPolygon(UpFolderBorderPen, new PointF[] {
                new PointF(
                    (float)drawXOffset - cwb - shb,
                    (float)drawYOffset + chb - swb),
                new PointF(
                    (float)drawXOffset - cwb + shb,
                    (float)drawYOffset - swb - chb),
                new PointF(
                    (float)drawXOffset + cwb + shb,
                    (float)drawYOffset + swb - chb),
                new PointF(
                    (float)drawXOffset + cwb - shb,
                    (float)drawYOffset + swb + chb)
            });

                g.DrawImage(folderImage, new PointF[] {
                new PointF(
                    (float)drawXOffset - cw + sh,
                    (float)drawYOffset - sw - ch),
                new PointF(
                    (float)drawXOffset + cw + sh,
                    (float)drawYOffset + sw - ch),
                new PointF(
                    (float)drawXOffset - cw - sh,
                    (float)drawYOffset + ch - sw)
            });
                folderImage.Dispose();
            }

            if (isParentFolder) {
                Pen UpArrowPen = new Pen(new SolidBrush(UpArrowColor), UpArrowWidth);
                UpArrowPen.LineJoin = LineJoin.Round;
                UpArrowPen.StartCap = LineCap.Flat;
                UpArrowPen.EndCap = LineCap.Round;

                g.DrawLines(UpArrowPen, new PointF[] {
                new PointF((float)drawXOffset - UpArrowSize * size, (float)drawYOffset + UpArrowSize * size),
                new PointF((float)drawXOffset + UpArrowSize * size, (float)drawYOffset + UpArrowSize * size),
                new PointF((float)drawXOffset + UpArrowSize * size, (float)drawYOffset - UpArrowSize * size),
                new PointF((float)drawXOffset + UpArrowSize * size * 2 / 3, (float)drawYOffset - UpArrowSize * size * 2 / 3),
                new PointF((float)drawXOffset + UpArrowSize * size, (float)drawYOffset - UpArrowSize * size),
                new PointF((float)drawXOffset + UpArrowSize * size * 4 / 3, (float)drawYOffset - UpArrowSize * size * 2 / 3)
            });
            }

            g.Dispose();
            return newImage;
        }

        /// <summary>
        /// Gets the JPEG codec.
        /// </summary>
        /// <returns>The JPEG codec.</returns>
        public ImageCodecInfo GetImageCodec()
        {
            ImageCodecInfo codec = null;
            ImageCodecInfo[] encoders = ImageCodecInfo.GetImageEncoders();

            foreach (ImageCodecInfo e in encoders)
            {
                if (e.MimeType == "image/jpeg")
                {
                    codec = e;
                    break;
                }
            }

            return codec;
        }

        /// <summary>
        /// Returns the encoder parameters.
        /// </summary>
        /// <returns></returns>
        public EncoderParameters GetImageEncoderParams()
        {
            EncoderParameters encoderParams = new EncoderParameters();
            encoderParams.Param[0] = new EncoderParameter(Encoder.Quality, ThumbnailJpegQuality);
            return encoderParams;
        }

        /// <summary>
        /// Gets an array of bytes from a Bitmap.
        /// </summary>
        /// <param name="image">The Bitmap.</param>
        /// <returns>The byte array.</returns>
        public byte[] GetImageBytes(Bitmap image)
        {
            ImageCodecInfo codec = GetImageCodec();
            EncoderParameters encoderParams = GetImageEncoderParams();

            MemoryStream ms = new MemoryStream();
            image.Save(ms, codec, encoderParams);
            ms.Close();

            encoderParams.Dispose();
            return ms.ToArray();
        }

    }
 
}